clear

a  = 1;
b  = 2.5;
y0 = 0;
Nvec  = [10,20,40,80];

err_FE  = zeros(1,length(Nvec));
err_AH  = zeros(1,length(Nvec));
err_RK2 = zeros(1,length(Nvec));
err_RK4 = zeros(1,length(Nvec));

for icount = 1:length(Nvec)

  N = Nvec(icount);
  h = (b-a)/N;

  % Construct an approximate solution using a GOOD ode solver.
  opts = odeset('AbsTol',1e-10,'RelTol',1e-10);
  [tt,yy] = ode45('odefun1',[a,b],y0,opts);
  y_at_b = yy(end);

  % Construct an approximation using forwards Euler.
  tt = linspace(a,b,N+1);
  yy = zeros(size(tt));
  yy(1) = y0;
  for n = 1:N
    yy(n+1) = yy(n) + h*odefun1(tt(n),yy(n));
  end
  err_FE(icount) = abs(yy(end) - y_at_b);
  
  % Construct an approximation using the 2nd order forwards scheme.
  tt = linspace(a,b,N+1);
  yy = zeros(size(tt));
  yy(1) = y0;
  yy(2) = y0 + h*odefun1(tt(1),yy(1));
  for n = 2:N
    yy(n+1) = yy(n-1) + 2*h*odefun1(tt(n),yy(n));
  end
  err_AH(icount) = abs(yy(end) - y_at_b);
  
  % Construct an approximation using the 2nd order Runge-Kutta.
  tt = linspace(a,b,N+1);
  yy = zeros(size(tt));
  yy(1) = y0;
  for n = 1:N
    k1 = odefun1(tt(n),yy(n));
    k2 = odefun1(tt(n) + 0.5*h, yy(n) + 0.5*h*k1);
    yy(n+1) = yy(n) + h*k2;
  end
  err_RK2(icount) = abs(yy(end) - y_at_b);
  
  % Construct an approximation using the 4th order Runge-Kutta.
  tt = linspace(a,b,N+1);
  yy = zeros(size(tt));
  yy(1) = y0;
  for n = 1:N
    k1 = odefun1(tt(n),yy(n));
    k2 = odefun1(tt(n) + 0.5*h, yy(n) + 0.5*h*k1);
    k3 = odefun1(tt(n) + 0.5*h, yy(n) + 0.5*h*k2);
    k4 = odefun1(tt(n) + h, yy(n) + h*k3);
    yy(n+1) = yy(n) + (h/6)*(k1 + 2*k2 + 2*k3 + k4);
  end
  err_RK4(icount) = abs(yy(end) - y_at_b);
  
end

disp(sprintf('h            | Forwards Euler         Ad hoc  Runge-Kutta 2  Runge-Kutta 4'))
for icount = 1:length(Nvec)
  disp(sprintf('%10.3e   |     %10.3e     %10.3e     %10.3e     %10.3e',...
               (b-a)/Nvec(icount),err_FE(icount),err_AH(icount),err_RK2(icount),err_RK4(icount)))  
end

loglog(Nvec,err_FE,Nvec,err_AH,Nvec,err_RK2,Nvec,err_RK4)
axis tight
legend('FE','AH','RK2','RK4')
xlabel('N')
ylabel('Error')

%loglog((b-a)./Nvec,err_FE,(b-a)./Nvec,err_AH,(b-a)./Nvec,err_RK2,(b-a)./Nvec,err_RK4)