% clear

a  = -4;
b  = 1;
y0 = -1.8;
N  = 80;
h  = (b-a)/N;

% Draw the gradient field.
nqt  = 20;
nqy  = 20;
ymin = -3;
ymax = 3;
[tt,yy] = meshgrid(linspace(a-0.5,b+0.5,nqt),linspace(ymin,ymax,nqy));
hold off
quiver(tt,yy,ones(nqt,nqy),odefun3(tt,yy),'g')
hold on
axis equal

% Construct an approximate solution using a GOOD ode solver.
opts = odeset('AbsTol',1e-10);
[tt,yy] = ode45('odefun3',[a,b],y0,opts);
plot(tt,yy,'k')
axis([a-0.5,b+0.5,ymin,ymax])
y_at_b = yy(end);

% Construct an approximation using forwards Euler.
tt = linspace(a,b,N+1);
yy = zeros(size(tt));
yy(1) = y0;
for n = 1:N
  yy(n+1) = yy(n) + h*odefun3(tt(n),yy(n));
end
plot(tt,yy,'r-x')
disp(sprintf('Error in forwards Euler             = %g',abs(yy(end) - y_at_b)))

% Construct an approximation using the 2nd order forwards scheme.
tt = linspace(a,b,N+1);
yy = zeros(size(tt));
yy(1) = y0;
yy(2) = y0 + h*odefun3(tt(1),yy(1));
for n = 2:N
  yy(n+1) = yy(n-1) + 2*h*odefun3(tt(n),yy(n));
end
plot(tt,yy,'b-x')
disp(sprintf('Error in ad hoc second order scheme = %g',abs(yy(end) - y_at_b)))

% Construct an approximation using the 2nd order Runge-Kutta.
tt = linspace(a,b,N+1);
yy = zeros(size(tt));
yy(1) = y0;
for n = 1:N
  k1 = odefun3(tt(n),yy(n));
  k2 = odefun3(tt(n) + 0.5*h, yy(n) + 0.5*h*k1);
  yy(n+1) = yy(n) + h*k2;
end
plot(tt,yy,'m-x')
disp(sprintf('Error in second order Runge-Kutta   = %g',abs(yy(end) - y_at_b)))

% Construct an approximation using the 4th order Runge-Kutta.
tt = linspace(a,b,N+1);
yy = zeros(size(tt));
yy(1) = y0;
for n = 1:N
  k1 = odefun3(tt(n),yy(n));
  k2 = odefun3(tt(n) + 0.5*h, yy(n) + 0.5*h*k1);
  k3 = odefun3(tt(n) + 0.5*h, yy(n) + 0.5*h*k2);
  k4 = odefun3(tt(n) + h, yy(n) + h*k3);
  yy(n+1) = yy(n) + (h/6)*(k1 + 2*k2 + 2*k3 + k4);
end
plot(tt,yy,'c-x')
disp(sprintf('Error in fourth order Runge-Kutta   = %g',abs(yy(end) - y_at_b)))

hold off